@@ -3,12 +3,14 @@
 
 #ifdef USE_GNOME
 #include <gnome.h>
+#include <bonobo.h>
 #endif
 
 #include "gb.h"
 #include "gbwidget.h"
 #include "source.h"
 #include "glade_gnome.h"
+#include "glade_atk.h"
 
 /* These are used for outputting signal handler prototypes. */
 #define GB_PARAM_INDENT		40
@@ -40,7 +42,7 @@ gb_widget_write_source (GtkWidget * widget,
   GtkWidget *parent;
   GbWidget *gbwidget;
   GladeWidgetData *widget_data;
-  gchar *class_id, *child_name;
+  gchar *class_id, *child_name, *widget_name = NULL, *real_widget_name = NULL;
   gint source_len;
 
   /* This is a temporary(?) kludge so that the code for GtkDialogs is OK.
@@ -55,14 +57,16 @@ gb_widget_write_source (GtkWidget * widget,
   child_name = gb_widget_get_child_name (widget);
   if (child_name && data->create_widget)
     {
-      if (!strcmp (child_name, "Dialog:action_area")
-	  || !strcmp (child_name, "GnomeDialog:action_area"))
+      if (!strcmp (child_name, GladeChildDialogActionArea))
 	return;
     }
 
   parent = data->parent;
 
   class_id = gb_widget_get_class_id (widget);
+#if 0
+  g_print ("class_id: %s\n", class_id ? class_id : "");
+#endif
   data->write_children = TRUE;
 
   widget_data = gtk_object_get_data (GTK_OBJECT (widget), GB_WIDGET_DATA_KEY);
@@ -80,6 +84,8 @@ gb_widget_write_source (GtkWidget * widget,
 	}
       else
 	{
+	  /* This stores newly-created widgets, which we use to get default
+	     values from. Though we don't use them much at present. */
 	  data->standard_widget = (GtkWidget *) g_hash_table_lookup (data->standard_widgets, class_id);
 	  if (data->standard_widget == NULL)
 	    {
@@ -92,24 +98,31 @@ gb_widget_write_source (GtkWidget * widget,
 	      else if (!strcmp (class_id, "GnomeDateEdit"))
 		data->standard_widget = gnome_date_edit_new ((time_t) 0, TRUE,
 							     TRUE);
-	      else
-		data->standard_widget = GTK_WIDGET (gtk_object_newv (gtk_type_from_name (class_id), 0, NULL));
+	      /* We don't create standard widgets for Bonobo controls, for
+		 now. There may be a better way to get default values for
+		 them. */
+	      else if (!BONOBO_IS_WIDGET (widget))
+		data->standard_widget = GTK_WIDGET (g_object_newv (gtk_type_from_name (class_id), 0, NULL));
 #else
-	      data->standard_widget = GTK_WIDGET (gtk_object_newv (gtk_type_from_name (class_id), 0, NULL));
+	      data->standard_widget = GTK_WIDGET (g_object_newv (gtk_type_from_name (class_id), 0, NULL));
 #endif
-	      g_hash_table_insert (data->standard_widgets, class_id,
-				   data->standard_widget);
+
+	      if (data->standard_widget)
+		g_hash_table_insert (data->standard_widgets, class_id,
+				     data->standard_widget);
 	    }
 	}
 
-      data->real_wname = source_create_valid_identifier (gtk_widget_get_name (widget));
+      real_widget_name = source_create_valid_identifier (gtk_widget_get_name (widget));
       if (data->use_component_struct && widget_data->public_field)
-	data->wname = g_strdup_printf ("%s->%s", data->component_name,
-				       data->real_wname);
+	widget_name = g_strdup_printf ("%s->%s", data->component_name,
+				       real_widget_name);
       else
-	data->wname = data->real_wname;
+	widget_name = real_widget_name;
 
       data->widget_data = widget_data;
+      data->real_wname = real_widget_name;
+      data->wname = widget_name;
       if (gbwidget->gb_widget_write_source)
 	(gbwidget->gb_widget_write_source) (widget, data);
       else
@@ -122,9 +135,6 @@ gb_widget_write_source (GtkWidget * widget,
 	      || data->source_buffers[GLADE_SOURCE]->str[source_len - 2] != '\n'))
 	source_add (data, "\n");
 
-      if (data->wname != data->real_wname)
-	g_free (data->real_wname);
-      g_free (data->wname);
       data->wname = NULL;
       data->real_wname = NULL;
       data->parent = widget;
@@ -143,7 +153,7 @@ gb_widget_write_source (GtkWidget * widget,
 	      /* Make sure the dummy widget is declared. */
 	      source_ensure_decl (data, "  GtkWidget *empty_notebook_page;\n");
 
-	      parent_name = gtk_widget_get_name (parent);
+	      parent_name = (char*) gtk_widget_get_name (parent);
 	      parent_name = source_create_valid_identifier (parent_name);
 	      source_add (data,
 			  "  %s = gtk_vbox_new (FALSE, 0);\n"
@@ -188,14 +198,32 @@ gb_widget_write_source (GtkWidget * widget,
      so that they work OK for the next sibling. */
   data->parent = parent;
 
+  /* SPECIAL CODE: For GtkOptionMenu, we have to set the menu after all the
+     children are created. */
+  if (GTK_IS_OPTION_MENU (widget) && GB_IS_GB_WIDGET (widget)
+      && GTK_OPTION_MENU (widget)->menu
+      && GB_IS_GB_WIDGET (GTK_OPTION_MENU (widget)->menu))
+    {
+      gchar *menu_name = source_create_valid_identifier (gtk_widget_get_name (GTK_OPTION_MENU (widget)->menu));
+      source_add (data,
+		  "  gtk_option_menu_set_menu (GTK_OPTION_MENU (%s), %s);\n\n",
+		  widget_name, menu_name);
+      g_free (menu_name);
+    }
+
   /* SPECIAL CODE: Finish off the GnomeUIInfo struct if we are building a
      Gnome menu. */
 #ifdef USE_GNOME
-  if (data->project->gnome_support && GTK_IS_MENU_SHELL (widget))
+  if (data->project->gnome_support && GTK_IS_MENU_SHELL (widget)
+      && GB_IS_GB_WIDGET (widget))
     {
       glade_gnome_finish_menu_source (GTK_MENU_SHELL (widget), data);
     }
 #endif
+
+  if (widget_name != real_widget_name)
+    g_free (real_widget_name);
+  g_free (widget_name);
 }
 
 
@@ -225,22 +253,19 @@ gb_widget_write_standard_source (GtkWidget * widget,
 	 gets destroyed. This may be a bug in GTK+ 1.2.3. */
       if (data->parent == NULL)
 	{
-	  source_add (data,
-		      "  gtk_object_set_data (GTK_OBJECT (%s), %s, %s);\n",
-		      data->component_name,
-		      source_make_string (data->real_wname, FALSE),
-		      data->wname);
+	  source_add_to_buffer (data, GLADE_OBJECT_HOOKUP,
+				"  GLADE_HOOKUP_OBJECT_NO_REF (%s, %s, %s);\n",
+				data->component_name,
+				data->wname,
+				source_make_string (data->real_wname, FALSE));
 	}
       else
 	{
-	  source_add (data,
-		      "  gtk_widget_ref (%s);\n"
-		      "  gtk_object_set_data_full (GTK_OBJECT (%s), %s, %s,\n"
-		      "                            (GtkDestroyNotify) gtk_widget_unref);\n",
-		      data->wname,
-		      data->component_name,
-		      source_make_string (data->real_wname, FALSE),
-		      data->wname);
+	  source_add_to_buffer (data, GLADE_OBJECT_HOOKUP,
+				"  GLADE_HOOKUP_OBJECT (%s, %s, %s);\n",
+				data->component_name,
+				data->wname,
+				source_make_string (data->real_wname, FALSE));
 	}
     }
 
@@ -261,19 +286,6 @@ gb_widget_write_standard_source (GtkWidget * widget,
   /* Output code to add widget to parent. */
   gb_widget_write_add_child_source (widget, data);
 
-  if (wdata->flags & (GLADE_X_SET | GLADE_Y_SET))
-    {
-      /* SPECIAL CODE: Widgets in a GtkLayout are positioned with
-	 gtk_layout_put(). */
-      if (!widget->parent || !GTK_IS_LAYOUT (widget->parent))
-	{
-	  source_add (data, "  gtk_widget_set_uposition (%s, %i, %i);\n",
-		      data->wname,
-		      wdata->flags & GLADE_X_SET ? wdata->x : -2,
-		      wdata->flags & GLADE_Y_SET ? wdata->y : -2);
-	}
-    }
-
   if (wdata->flags & (GLADE_WIDTH_SET | GLADE_HEIGHT_SET))
     {
       width = wdata->flags & GLADE_WIDTH_SET ? wdata->width : -2;
@@ -293,7 +305,7 @@ gb_widget_write_standard_source (GtkWidget * widget,
     }
 
 
-  if (GTK_IS_CONTAINER (widget))
+  if (glade_util_uses_border_width (widget))
     {
       if (GTK_CONTAINER (widget)->border_width != 0)
 	{
@@ -396,6 +408,7 @@ gb_widget_write_standard_source (GtkWidget * widget,
 
   gb_widget_write_signals_source (widget, data);
   gb_widget_write_accelerators_source (widget, data);
+  glade_atk_write_source (widget, data);
 }
 
 
@@ -422,7 +435,7 @@ gb_widget_write_add_child_source (GtkWidget * widget,
   if (!parent)
     return;
 
-  parent_name = gtk_widget_get_name (parent);
+  parent_name = (char*) gtk_widget_get_name (parent);
   parent_name = source_create_valid_identifier (parent_name);
 
   MSG2 ("Adding %s to %s", data->wname, parent_name);
@@ -531,7 +544,7 @@ gb_widget_write_signal_handler_source (GtkWidget *widget,
 				       const gchar *handler)
 {
   guint signal_id;
-  GtkSignalQuery *query_info;
+  GSignalQuery query_info;
   gint param, i;
   gchar buffer[1024];
   gchar *ret_type, *pos, *type_name, *arg_name, *object_name, *object_arg;
@@ -554,11 +567,10 @@ gb_widget_write_signal_handler_source (GtkWidget *widget,
   signal_id = gtk_signal_lookup (signal_name, GTK_OBJECT_TYPE (widget));
   g_return_if_fail (signal_id != 0);
 
-  query_info = gtk_signal_query (signal_id);
-  g_return_if_fail (query_info != NULL);
+  g_signal_query (signal_id, &query_info);
 
   /* Output the return type and function name. */
-  ret_type = get_type_name (query_info->return_val, &is_pointer);
+  ret_type = get_type_name (query_info.return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE, &is_pointer);
   pos = buffer;
   sprintf (pos, "%s%s\n%s", ret_type, is_pointer ? "*" : "", handler);
   pos += strlen (pos);
@@ -582,7 +594,7 @@ gb_widget_write_signal_handler_source (GtkWidget *widget,
   /* Output the signal object type and the argument name. We assume the
      type is a pointer - I think that is OK. We remove "Gtk" and convert
      to lower case for the argument name. */
-  object_name = gtk_type_name (query_info->object_type);
+  object_name = (char*) g_type_name (query_info.itype);
   sprintf (pos, "(%s ", object_name);
   pos += strlen (pos);
   object_name_len = strlen (object_name);
@@ -603,7 +615,7 @@ gb_widget_write_signal_handler_source (GtkWidget *widget,
   pos += strlen (pos);
 
   /* Convert signal name to use underscores rather than dashes '-'. */
-  signal_name_copy = g_strdup (query_info->signal_name);
+  signal_name_copy = g_strdup (query_info.signal_name);
   for (i = 0; signal_name_copy[i]; i++)
     {
       if (signal_name_copy[i] == '-')
@@ -615,7 +627,7 @@ gb_widget_write_signal_handler_source (GtkWidget *widget,
 
   param_num = 1;
   event_num = callback_num = 0;
-  for (param = 0; param < query_info->nparams; param++)
+  for (param = 0; param < query_info.n_params; param++)
     {
       for (i = 0; i < GB_PARAM_INDENT; i++)
 	*pos++ = ' ';
@@ -627,7 +639,7 @@ gb_widget_write_signal_handler_source (GtkWidget *widget,
 	}
       else
 	{
-	  type_name = get_type_name (query_info->params[param], &is_pointer);
+	  type_name = get_type_name (query_info.param_types[param] & ~G_SIGNAL_TYPE_STATIC_SCOPE, &is_pointer);
 	  /* Most arguments to the callback are called "arg1", "arg2", etc.
 	     GtkWidgets are called "widget", "widget2", ...
 	     GdkEvents are called "event", "event2", ...
@@ -690,11 +702,10 @@ gb_widget_write_signal_handler_source (GtkWidget *widget,
   source_add_to_buffer (data, GLADE_CALLBACK_SOURCE,
 			"\n%s\n{\n\n%s}\n\n",
 			buffer,
-			query_info->return_val == GTK_TYPE_BOOL
+			query_info.return_type == GTK_TYPE_BOOL
 			? "  return FALSE;\n" : "");
 
   g_free (signal_name_copy);
-  g_free (query_info);
 }
 
 
@@ -760,55 +771,68 @@ get_type_name (GtkType type, gboolean * is_pointer)
   gchar *type_name;
 
   *is_pointer = FALSE;
-  type_name = gtk_type_name (type);
+  type_name = (char*) g_type_name (type);
 
   switch (type) {
-  case GTK_TYPE_NONE:
-  case GTK_TYPE_CHAR:
-  case GTK_TYPE_UCHAR:
-  case GTK_TYPE_BOOL:
-  case GTK_TYPE_INT:
-  case GTK_TYPE_UINT:
-  case GTK_TYPE_LONG:
-  case GTK_TYPE_ULONG:
-  case GTK_TYPE_FLOAT:
-  case GTK_TYPE_DOUBLE:
-  case GTK_TYPE_POINTER:
+  case G_TYPE_NONE:
+  case G_TYPE_CHAR:
+  case G_TYPE_UCHAR:
+  case G_TYPE_BOOLEAN:
+  case G_TYPE_INT:
+  case G_TYPE_UINT:
+  case G_TYPE_LONG:
+  case G_TYPE_ULONG:
+  case G_TYPE_FLOAT:
+  case G_TYPE_DOUBLE:
+  case G_TYPE_POINTER:
     /* These all have normal C type names so they are OK. */
     return type_name;
 
-  case GTK_TYPE_STRING:
+  case G_TYPE_STRING:
     /* A GtkString is really a gchar*. */
     *is_pointer = TRUE;
     return "gchar";
 
-  case GTK_TYPE_ENUM:
-  case GTK_TYPE_FLAGS:
-    /* We use a gint for both of these. Hopefully a subclass with a decent
+  case G_TYPE_ENUM:
+  case G_TYPE_FLAGS:
+    /* We use a gint for both of these. Hopefully a subtype with a decent
        name will be registered and used instead, as GTK+ does itself. */
     return "gint";
 
-  case GTK_TYPE_BOXED:
-    /* A boxed value is just an opaque pointer, I think. */
-    return "gpointer";
-
-  case GTK_TYPE_SIGNAL:
-  case GTK_TYPE_ARGS:
-  case GTK_TYPE_FOREIGN:
-  case GTK_TYPE_CALLBACK:
-  case GTK_TYPE_C_CALLBACK:
-    /* FIXME: These are wrong. I think they expand into more than 1 argument.
-       See the GtkArg struct in gtktypeutils.h and gtkargcollector.c.
-       Fortunately I doubt anything uses these as signal args. */
+  case G_TYPE_BOXED:
+    /* The boxed type shouldn't be used itself, only subtypes. Though we
+       return 'gpointer' just in case. */
     return "gpointer";
 
   default:
     break;
   }
 
-  /* For all GtkObject subclasses we can use the class name with a "*",
+  if (!strcmp (type_name, "GtkTypeTextIter"))
+    {
+      *is_pointer = TRUE;
+      return "GtkTextIter";
+    }
+
+  if (!strcmp (type_name, "GtkTypeTreeIter"))
+    {
+      *is_pointer = TRUE;
+      return "GtkTreeIter";
+    }
+
+  if (!strcmp (type_name, "GtkTypeTreePath"))
+    {
+      *is_pointer = TRUE;
+      return "GtkTreePath";
+    }
+
+  /* For all GtkObject subtypes we can use the class name with a "*",
      e.g. 'GtkWidget *'. */
-  if (gtk_type_is_a (type, GTK_TYPE_OBJECT))
+  if (g_type_is_a (type, G_TYPE_OBJECT))
+    *is_pointer = TRUE;
+
+  /* All boxed subtypes will be pointers as well. */
+  if (g_type_is_a (type, G_TYPE_BOXED))
     *is_pointer = TRUE;
 
   return type_name;
