@@ -41,28 +41,31 @@ static GbWidget gbwidget;
 
 static gchar *Title = "GtkWindow::title";
 static gchar *Type = "GtkWindow::type";
-static gchar *Position = "GtkWindow::position";
+static gchar *Position = "GtkWindow::window_position";
 static gchar *Modal = "GtkWindow::modal";
 static gchar *DefaultWidth = "GtkWindow::default_width";
 static gchar *DefaultHeight = "GtkWindow::default_height";
-static gchar *Shrink = "GtkWindow::allow_shrink";
-static gchar *Grow = "GtkWindow::allow_grow";
-static gchar *AutoShrink = "GtkWindow::auto_shrink";
 static gchar *WMName = "GtkWindow::wmclass_name";
 static gchar *WMClass = "GtkWindow::wmclass_class";
+static gchar *Resizable = "GtkWindow::resizable";
+static gchar *DestroyWithParent = "GtkWindow::destroy_with_parent";
+static gchar *Icon = "GtkWindow::icon";
+
+/* These are deprecated. We don't show them any more. */
+static gchar *Shrink = "GtkWindow::allow_shrink";
+static gchar *Grow = "GtkWindow::allow_grow";
+
 
 static const gchar *GbTypeChoices[] =
-{"Top Level", "Dialog", "Popup", NULL};
+{"Top Level", "Popup", NULL};
 static const gint GbTypeValues[] =
 {
   GTK_WINDOW_TOPLEVEL,
-  GTK_WINDOW_DIALOG,
   GTK_WINDOW_POPUP
 };
 static const gchar *GbTypeSymbols[] =
 {
   "GTK_WINDOW_TOPLEVEL",
-  "GTK_WINDOW_DIALOG",
   "GTK_WINDOW_POPUP"
 };
 
@@ -130,8 +133,9 @@ gb_window_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
   gb_window_create_standard_properties (widget, data,
 					Title, Type, Position, Modal,
 					DefaultWidth, DefaultHeight,
-					Shrink, Grow, AutoShrink,
-					WMName, WMClass);
+					Shrink, Grow, NULL,
+					WMName, WMClass,
+					Resizable, DestroyWithParent, Icon);
 }
 
 
@@ -148,7 +152,10 @@ gb_window_create_standard_properties (GtkWidget * widget,
 				      gchar *grow_p,
 				      gchar *auto_shrink_p,
 				      gchar *wmname_p,
-				      gchar *wmclass_p)
+				      gchar *wmclass_p,
+				      gchar *resizable_p,
+				      gchar *destroy_with_parent_p,
+				      gchar *icon_p)
 {
   if (title_p)
     property_add_string (title_p, _("Title:"), _("The title of the window"));
@@ -179,15 +186,20 @@ gb_window_create_standard_properties (GtkWidget * widget,
 				       on_toggle_default_height);
     }
 
-  if (shrink_p && grow_p && auto_shrink_p)
+  if (resizable_p)
+    property_add_bool (resizable_p, _("Resizable:"),
+		       _("If the window can be resized"));
+
+#if 0
+  /* These are deprecated. */
+  if (shrink_p && grow_p)
     {
       property_add_bool (shrink_p, _("Shrink:"),
 			 _("If the window can be shrunk"));
       property_add_bool (grow_p, _("Grow:"),
 			 _("If the window can be enlarged"));
-      property_add_bool (auto_shrink_p, _("Auto Shrink:"),
-			 _("If the window shrinks automatically"));
     }
+#endif
 
   if (wmname_p)
     property_add_string (wmname_p, _("WM Name:"),
@@ -195,6 +207,14 @@ gb_window_create_standard_properties (GtkWidget * widget,
   if (wmclass_p)
     property_add_string (wmclass_p, _("WM Class:"),
 			 _("The class name to pass to the window manager"));
+
+  if (destroy_with_parent_p)
+    property_add_bool (destroy_with_parent_p, _("Auto-Destroy:"),
+		       _("If the window is destroyed when its transient parent is destroyed"));
+
+  if (icon_p)
+    property_add_filename (icon_p, _("Icon:"),
+			   _("The icon for this window"));
 }
 
 
@@ -256,8 +276,9 @@ gb_window_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
   gb_window_get_standard_properties (widget, data,
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
-				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass);
+				     Shrink, Grow, NULL,
+				     WMName, WMClass,
+				     Resizable, DestroyWithParent, Icon);
 }
 
 
@@ -274,7 +295,10 @@ gb_window_get_standard_properties (GtkWidget * widget,
 				   gchar *grow_p,
 				   gchar *auto_shrink_p,
 				   gchar *wmname_p,
-				   gchar *wmclass_p)
+				   gchar *wmclass_p,
+				   gchar *resizable_p,
+				   gchar *destroy_with_parent_p,
+				   gchar *icon_p)
 {
   gint type, position, default_width, default_height;
 
@@ -322,15 +346,16 @@ gb_window_get_standard_properties (GtkWidget * widget,
 				     default_height != 0 ? TRUE : FALSE);
     }
 
-  if (shrink_p && grow_p && auto_shrink_p)
+#if 0
+  /* These are deprecated. */
+  if (grow_p && shrink_p)
     {
-      gb_widget_output_bool (data, shrink_p,
-			     GTK_WINDOW (widget)->allow_shrink);
       gb_widget_output_bool (data, grow_p,
 			     GTK_WINDOW (widget)->allow_grow);
-      gb_widget_output_bool (data, auto_shrink_p,
-			     GTK_WINDOW (widget)->auto_shrink);
+      gb_widget_output_bool (data, shrink_p,
+			     GTK_WINDOW (widget)->allow_shrink);
     }
+#endif
 
   /* These are stored in the object hash since we can't set them after the
      window is realized. */
@@ -342,6 +367,19 @@ gb_window_get_standard_properties (GtkWidget * widget,
     gb_widget_output_string (data, wmclass_p,
 			     gtk_object_get_data (GTK_OBJECT (widget),
 						  wmclass_p));
+
+  if (resizable_p)
+    gb_widget_output_bool (data, resizable_p,
+			   gtk_window_get_resizable (GTK_WINDOW (widget)));
+
+  if (destroy_with_parent_p)
+    gb_widget_output_bool (data, destroy_with_parent_p,
+			   gtk_window_get_destroy_with_parent (GTK_WINDOW (widget)));
+
+  if (icon_p)
+    gb_widget_output_pixmap_filename (data, icon_p,
+				      gtk_object_get_data (GTK_OBJECT (widget),
+							   icon_p));
 }
 
 
@@ -356,8 +394,9 @@ gb_window_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
   gb_window_set_standard_properties (widget, data,
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
-				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass);
+				     Shrink, Grow, NULL,
+				     WMName, WMClass,
+				     Resizable, DestroyWithParent, Icon);
 }
 
 
@@ -374,12 +413,15 @@ gb_window_set_standard_properties (GtkWidget * widget,
 				   gchar *grow_p,
 				   gchar *auto_shrink_p,
 				   gchar *wmname_p,
-				   gchar *wmclass_p)
+				   gchar *wmclass_p,
+				   gchar *resizable_p,
+				   gchar *destroy_with_parent_p,
+				   gchar *icon_p)
 {
   gchar *title, *type, *position;
   gint default_width, default_height, i;
   gboolean modal, apply_default_width, apply_default_height;
-  gboolean shrink, grow, auto_shrink, set_policy = FALSE;
+  gboolean resizable, destroy_with_parent;
   gchar *wmname, *wmclass;
 
   if (title_p)
@@ -422,12 +464,6 @@ gb_window_set_standard_properties (GtkWidget * widget,
 		{
 		  gtk_object_set_data (GTK_OBJECT (widget), position_p,
 				       GINT_TO_POINTER (i));
-		  /* We may need to update the position properties to reflect
-		     the position setting (X & Y are only applicable when the
-		     position is set to GTK_WIN_POS_NONE). */
-		  if (GbPositionValues[i] != GTK_WIN_POS_NONE)
-		    data->widget_data->flags &= ~(GB_X_SET | GB_Y_SET);
-		  gb_widget_show_position_properties (widget);
 		  break;
 		}
 	    }
@@ -476,7 +512,9 @@ gb_window_set_standard_properties (GtkWidget * widget,
 	}
     }
 
-  if (shrink_p && grow_p && auto_shrink_p)
+#if 0
+  /* These are deprecated. */
+  if (shrink_p && grow_p)
     {
       shrink = gb_widget_input_bool (data, shrink_p);
       if (data->apply)
@@ -490,16 +528,10 @@ gb_window_set_standard_properties (GtkWidget * widget,
       else
 	grow = GTK_WINDOW (widget)->allow_grow;
 
-      auto_shrink = gb_widget_input_bool (data, auto_shrink_p);
-      if (data->apply)
-	set_policy = TRUE;
-      else
-	auto_shrink = GTK_WINDOW (widget)->auto_shrink;
-
       if (set_policy)
-	gtk_window_set_policy (GTK_WINDOW (widget), shrink, grow, auto_shrink);
+	gtk_window_set_policy (GTK_WINDOW (widget), shrink, grow, FALSE);
     }
-
+#endif
 
   if (wmname_p)
     {
@@ -525,6 +557,53 @@ gb_window_set_standard_properties (GtkWidget * widget,
 				    wmclass ? g_free : NULL);
 	}
     }
+
+  if (resizable_p)
+    {
+      resizable = gb_widget_input_bool (data, resizable_p);
+      if (data->apply)
+	gtk_window_set_resizable (GTK_WINDOW (widget), resizable);
+    }
+
+  if (destroy_with_parent_p)
+    {
+      destroy_with_parent = gb_widget_input_bool (data, destroy_with_parent_p);
+      if (data->apply)
+	gtk_window_set_destroy_with_parent (GTK_WINDOW (widget),
+					    destroy_with_parent);
+    }
+
+  if (icon_p)
+    {
+      char *filename = gb_widget_input_pixmap_filename (data, icon_p);
+      if (data->apply)
+	{
+	  char *old_filename;
+
+	  if (filename && filename[0] == '\0')
+	    filename = NULL;
+
+	  /* Remove the old pixmap from the project. */
+	  old_filename = gtk_object_get_data (GTK_OBJECT (widget), icon_p);
+	  glade_project_remove_pixmap (data->project, old_filename);
+
+	  gtk_object_set_data_full (GTK_OBJECT (widget), icon_p,
+				    g_strdup (filename),
+				    filename ? g_free : NULL);
+
+	  glade_project_add_pixmap (data->project, filename);
+
+	  if (filename)
+	    {
+	      GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
+	      gtk_window_set_icon (GTK_WINDOW (widget), pixbuf);
+	      if (pixbuf)
+		gdk_pixbuf_unref (pixbuf);
+	    }
+	}
+      if (data->action == GB_LOADING)
+	g_free (filename);
+    }
 }
 
 
@@ -567,8 +646,9 @@ gb_window_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
   gb_window_write_standard_source (widget, data,
 				   Title, NULL, Position, Modal,
 				   DefaultWidth, DefaultHeight,
-				   Shrink, Grow, AutoShrink,
-				   WMName, WMClass);
+				   Shrink, Grow, NULL,
+				   WMName, WMClass,
+				   Resizable, DestroyWithParent, Icon);
 }
 
 
@@ -585,14 +665,18 @@ gb_window_write_standard_source (GtkWidget * widget,
 				 gchar *grow_p,
 				 gchar *auto_shrink_p,
 				 gchar *wmname_p,
-				 gchar *wmclass_p)
+				 gchar *wmclass_p,
+				 gchar *resizable_p,
+				 gchar *destroy_with_parent_p,
+				 gchar *icon_p)
 {
   gint type, position, default_width, default_height;
   gchar *wmname, *wmclass;
 
   if (title_p)
     {
-      if (strlen (GTK_WINDOW (widget)->title) > 0)
+      if (GTK_WINDOW (widget)->title
+	  && strlen (GTK_WINDOW (widget)->title) > 0)
 	source_add (data, "  gtk_window_set_title (GTK_WINDOW (%s), %s);\n",
 		    data->wname,
 		    source_make_string (GTK_WINDOW (widget)->title,
@@ -649,18 +733,19 @@ gb_window_write_standard_source (GtkWidget * widget,
 	}
     }
 
-  if (shrink_p && grow_p && auto_shrink_p)
+#if 0
+  if (shrink_p && grow_p)
     {
-      if (GTK_WINDOW (widget)->allow_shrink != FALSE
-	  || GTK_WINDOW (widget)->allow_grow != TRUE
-	  || GTK_WINDOW (widget)->auto_shrink != FALSE)
+      if (GTK_WINDOW (widget)->allow_grow != TRUE
+	  || GTK_WINDOW (widget)->allow_shrink != FALSE)
 	source_add (data,
 		    "  gtk_window_set_policy (GTK_WINDOW (%s), %s, %s, %s);\n",
 		    data->wname,
 		    GTK_WINDOW (widget)->allow_shrink ? "TRUE" : "FALSE",
 		    GTK_WINDOW (widget)->allow_grow ? "TRUE" : "FALSE",
-		    GTK_WINDOW (widget)->auto_shrink ? "TRUE" : "FALSE");
+		    "FALSE");
     }
+#endif
 
   if (wmname_p && wmclass_p)
     {
@@ -676,6 +761,43 @@ gb_window_write_standard_source (GtkWidget * widget,
 		      wmclass ? source_make_string (wmclass, FALSE) : "\"\"");
 	}
     }
+
+  if (resizable_p)
+    {
+      if (!gtk_window_get_resizable (GTK_WINDOW (widget)))
+	source_add (data,
+		    "  gtk_window_set_resizable (GTK_WINDOW (%s), FALSE);\n",
+		    data->wname);
+    }
+
+  if (destroy_with_parent_p)
+    {
+      if (gtk_window_get_destroy_with_parent (GTK_WINDOW (widget)))
+	source_add (data,
+	"  gtk_window_set_destroy_with_parent (GTK_WINDOW (%s), TRUE);\n",
+		    data->wname);
+    }
+
+  if (icon_p)
+    {
+      gchar *filename = gtk_object_get_data (GTK_OBJECT (widget), icon_p);
+      if (filename && *filename)
+	{
+	  char *pixbuf_name = g_strdup_printf ("%s_icon_pixbuf", data->wname);
+
+	  source_add_decl (data, "  GdkPixbuf *%s;\n", pixbuf_name);
+	  source_create_pixbuf (data, pixbuf_name, filename);
+	  source_add (data,
+		      "  if (%s)\n"
+		      "    {\n"
+		      "      gtk_window_set_icon (GTK_WINDOW (%s), %s);\n"
+		      "      gdk_pixbuf_unref (%s);\n"
+		      "    }\n",
+		      pixbuf_name, pixbuf_name, pixbuf_name, pixbuf_name);
+
+	  g_free (pixbuf_name);
+	}
+    }
 }
 
 
@@ -688,7 +810,8 @@ GbWidget *
 gb_window_init ()
 {
   /* Initialise the GTK type */
-  gtk_window_get_type ();
+  volatile GtkType type;
+  type = gtk_window_get_type ();
 
   /* Initialize the GbWidget structure */
   gb_widget_init_struct (&gbwidget);
