@@ -28,10 +28,6 @@
 #include <gtk/gtkvbox.h>
 #include <gtk/gtksignal.h>
 
-#ifdef USE_GNOME
-#include <libgnomeui/gnome-preferences.h>
-#endif
-
 #include "graphics/selector.xpm"
 #include "glade_palette.h"
 #include "glade.h"
@@ -106,19 +102,17 @@ glade_palette_class_init (GladePaletteClass * klass)
   glade_palette_signals[SELECT_ITEM] =
     gtk_signal_new ("select_item",
 		    GTK_RUN_LAST,
-		    object_class->type,
+		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GladePaletteClass, select_item),
-		    gtk_marshal_NONE__POINTER,
+		    gtk_marshal_VOID__STRING,
 		    GTK_TYPE_NONE, 1, GTK_TYPE_STRING);
   glade_palette_signals[UNSELECT_ITEM] =
     gtk_signal_new ("unselect_item",
 		    GTK_RUN_LAST,
-		    object_class->type,
+		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GladePaletteClass, unselect_item),
-		    gtk_marshal_NONE__POINTER,
+		    gtk_marshal_VOID__STRING,
 		    GTK_TYPE_NONE, 1, GTK_TYPE_STRING);
-  gtk_object_class_add_signals (object_class, glade_palette_signals,
-				LAST_SIGNAL);
 
   klass->select_item = NULL;
   klass->unselect_item = NULL;
@@ -144,24 +138,11 @@ glade_palette_init (GladePalette * palette)
   gtk_container_add (GTK_CONTAINER (palette), palette->vbox);
   gtk_widget_show (palette->vbox);
 
-  palette->notebook = gtk_notebook_new ();
-  /* We use gtk_box_pack_end, so we can later add buttons for new categories
-     at the right place with gtk_box_pack_start */
-  gtk_box_pack_end (GTK_BOX (palette->vbox), palette->notebook,
-		    FALSE, TRUE, 0);
-  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (palette->notebook), FALSE);
-  gtk_widget_show (palette->notebook);
-
   hbox = gtk_hbox_new (FALSE, 2);
   gtk_box_pack_start (GTK_BOX (palette->vbox), hbox, FALSE, TRUE, 0);
   gtk_widget_show (hbox);
 
-#ifdef USE_GNOME
-  if (gnome_preferences_get_toolbar_relief_btn())
-    relief_style = GTK_RELIEF_NORMAL;
-  else
-#endif
-    relief_style = GTK_RELIEF_NONE;
+  relief_style = GTK_RELIEF_NONE;
   
   palette->selector = gtk_radio_button_new (NULL);
   gtk_button_set_relief (GTK_BUTTON (palette->selector), relief_style);
@@ -197,6 +178,15 @@ glade_palette_init (GladePalette * palette)
   gtk_box_pack_start (GTK_BOX (palette->vbox), seperator, FALSE, TRUE, 3);
   gtk_widget_show (seperator);
 
+  palette->notebook = gtk_notebook_new ();
+  /* We use gtk_box_pack_end, so we can later add buttons for new categories
+     at the right place with gtk_box_pack_start */
+  gtk_box_pack_end (GTK_BOX (palette->vbox), palette->notebook,
+		    TRUE, TRUE, 0);
+  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (palette->notebook), FALSE);
+  gtk_notebook_set_show_border (GTK_NOTEBOOK (palette->notebook), FALSE);
+  gtk_widget_show (palette->notebook);
+
   seperator = gtk_hseparator_new ();
   gtk_box_pack_end (GTK_BOX (palette->vbox), seperator, FALSE, TRUE, 3);
   gtk_widget_show (seperator);
@@ -226,17 +216,11 @@ glade_palette_add_widget (GladePalette *palette,
   g_return_if_fail (palette != NULL);
   g_return_if_fail (GLADE_IS_PALETTE (palette));
   g_return_if_fail (gdkpixmap != NULL);
-  g_return_if_fail (mask != NULL);
 
   if (!(sect = g_hash_table_lookup (palette->sections, section)))
     sect = new_section (palette, section);
 
-#ifdef USE_GNOME
-  if (gnome_preferences_get_toolbar_relief_btn())
-    relief_style = GTK_RELIEF_NORMAL;
-  else
-#endif
-    relief_style = GTK_RELIEF_NONE;
+  relief_style = GTK_RELIEF_NONE;
 
   button = gtk_radio_button_new (palette->widget_button_group);
   gtk_button_set_relief (GTK_BUTTON (button), relief_style);
@@ -306,8 +290,9 @@ new_section (GladePalette * palette,
   gtk_notebook_append_page (GTK_NOTEBOOK (palette->notebook), sect->table,
 			    NULL);
   sect->page = g_list_length (GTK_NOTEBOOK (palette->notebook)->children) - 1;
-  button = gtk_radio_button_new_with_label (palette->section_button_group,
-					    section);
+  button = gtk_radio_button_new_with_mnemonic (palette->section_button_group,
+					       section);
+  gtk_widget_set_name (button, section);
   gtk_widget_set_usize (button, 20, -1);
   gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
   palette->section_button_group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
@@ -315,7 +300,11 @@ new_section (GladePalette * palette,
   gtk_object_set_data (GTK_OBJECT (button), "section", sect);
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
 		      GTK_SIGNAL_FUNC (on_section_button_clicked), palette);
-  gtk_box_pack_start (GTK_BOX (palette->vbox), button, FALSE, TRUE, 0);
+  /* Place the deprecated section last. */
+  if (!strcmp (section, _("Dep_recated")))
+    gtk_box_pack_end (GTK_BOX (palette->vbox), button, FALSE, TRUE, 0);
+  else
+    gtk_box_pack_start (GTK_BOX (palette->vbox), button, FALSE, TRUE, 0);
   g_hash_table_insert (palette->sections, (gpointer) section, sect);
   return sect;
 }
@@ -411,7 +400,7 @@ on_section_button_clicked (GtkWidget * button,
 
   sect = (GladePaletteSection*) gtk_object_get_data (GTK_OBJECT (button),
 						     "section");
-  gtk_notebook_set_page (GTK_NOTEBOOK (palette->notebook), sect->page);
+  gtk_notebook_set_current_page (GTK_NOTEBOOK (palette->notebook), sect->page);
 }
 
 
