@@ -23,6 +23,8 @@
 #endif
 #include <errno.h>
 
+#include <libxml/parser.h>
+
 #include <gtk/gtkmenu.h>
 
 #include "gladeconfig.h"
@@ -60,8 +62,8 @@ static void free_key (gchar	*key,
 		      gchar	*value,
 		      gpointer	 data);
 
-static const gchar* glade_project_find_id (GladeProject *project,
-					   const gchar *name);
+static gchar* glade_project_find_id (GladeProject *project,
+				     const gchar *name);
 static void glade_project_real_ensure_widgets_named (GtkWidget    *widget,
 						     GladeProject *project);
 
@@ -116,26 +118,24 @@ glade_project_class_init (GladeProjectClass * klass)
   glade_project_signals[ADD_COMPONENT] =
     gtk_signal_new ("add_component",
 		    GTK_RUN_LAST,
-		    object_class->type,
+		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GladeProjectClass, add_component),
-		    gtk_marshal_NONE__POINTER,
+		    gtk_marshal_VOID__OBJECT,
 		    GTK_TYPE_NONE, 1, GTK_TYPE_OBJECT);
   glade_project_signals[REMOVE_COMPONENT] =
     gtk_signal_new ("remove_component",
 		    GTK_RUN_LAST,
-		    object_class->type,
+		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GladeProjectClass, remove_component),
-		    gtk_marshal_NONE__POINTER,
+		    gtk_marshal_VOID__OBJECT,
 		    GTK_TYPE_NONE, 1, GTK_TYPE_OBJECT);
   glade_project_signals[COMPONENT_CHANGED] =
     gtk_signal_new ("component_changed",
 		    GTK_RUN_LAST,
-		    object_class->type,
+		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GladeProjectClass, component_changed),
-		    gtk_marshal_NONE__POINTER,
+		    gtk_marshal_VOID__OBJECT,
 		    GTK_TYPE_NONE, 1, GTK_TYPE_OBJECT);
-  gtk_object_class_add_signals (object_class, glade_project_signals,
-				LAST_SIGNAL);
 
   klass->add_component = NULL;
   klass->remove_component = NULL;
@@ -170,12 +170,12 @@ glade_project_init (GladeProject * project)
 
   project->current_directory = NULL;
 
+  project->gettext_support = TRUE;
+
 #ifdef USE_GNOME
   project->gnome_support = TRUE;
-  project->gettext_support = TRUE;
 #else
   project->gnome_support = FALSE;
-  project->gettext_support = FALSE;
 #endif
 
   project->gnome_db_support = FALSE;
@@ -231,11 +231,22 @@ glade_project_destroy (GtkObject *object)
   project = GLADE_PROJECT (object);
 
   g_free (project->xml_filename);
+  project->xml_filename = NULL;
 
   g_free (project->name);
+  project->name = NULL;
+
+  g_free (project->program_name);
+  project->program_name = NULL;
+
   g_free (project->directory);
+  project->directory = NULL;
+
   g_free (project->source_directory);
+  project->source_directory = NULL;
+
   g_free (project->pixmaps_directory);
+  project->pixmaps_directory = NULL;
 
   tree_clear ();
 
@@ -247,10 +258,15 @@ glade_project_destroy (GtkObject *object)
       tmp_list = tmp_list->next;
     }
   g_list_free (project->components);
- 
+  project->components = NULL;
+
   /* Destroy the unique id hash. */
-  g_hash_table_foreach (project->unique_id_hash, (GHFunc) free_key, NULL);
-  g_hash_table_destroy (project->unique_id_hash);
+  if (project->unique_id_hash)
+    {
+      g_hash_table_foreach (project->unique_id_hash, (GHFunc) free_key, NULL);
+      g_hash_table_destroy (project->unique_id_hash);
+      project->unique_id_hash = NULL;
+    }
 
   /* Free all pixmap filenames. */
   tmp_list = project->pixmap_filenames;
@@ -260,16 +276,28 @@ glade_project_destroy (GtkObject *object)
       tmp_list = tmp_list->next;
     }
   g_list_free (project->pixmap_filenames);
+  project->pixmap_filenames = NULL;
 
   g_free (project->current_directory);
+  project->current_directory = NULL;
 
   g_free (project->main_source_file);
+  project->main_source_file = NULL;
+
   g_free (project->main_header_file);
+  project->main_header_file = NULL;
+
   g_free (project->handler_source_file);
+  project->handler_source_file = NULL;
+
   g_free (project->handler_header_file);
+  project->handler_header_file = NULL;
 
   g_free (project->support_source_file);
+  project->support_source_file = NULL;
+
   g_free (project->support_header_file);
+  project->support_header_file = NULL;
 }
 
 
@@ -284,12 +312,11 @@ free_key (gchar		*key,
 }
 
 
-GladeStatusCode
+gboolean
 glade_project_open	(const gchar   *xml_filename,
-			 GladeProject **project_return,
-			 GList	      **errors)
+			 GladeProject **project_return)
 {
-  GladeStatusCode status;
+  gboolean status;
   GladeProject *project;
 
   project = glade_project_new ();
@@ -297,11 +324,11 @@ glade_project_open	(const gchar   *xml_filename,
 
   tree_freeze ();
 
-  status = load_project_file (project, errors);
+  status = load_project_file (project);
   
   tree_thaw ();
 
-  if (status != GLADE_STATUS_OK)
+  if (!status)
     {
       gtk_object_destroy (GTK_OBJECT (project));
       current_project = NULL;
@@ -790,9 +817,14 @@ void
 glade_project_add_pixmap		(GladeProject  *project,
 					 const gchar   *filename)
 {
-  project->pixmap_filenames = g_list_prepend (project->pixmap_filenames,
-					      g_strdup (filename));
-  glade_project_set_changed (project, TRUE);
+  if (filename && filename[0])
+    {
+      g_print ("Adding pixmap: %s\n", filename);
+
+      project->pixmap_filenames = g_list_prepend (project->pixmap_filenames,
+						  g_strdup (filename));
+      glade_project_set_changed (project, TRUE);
+    }
 }
 
 
@@ -804,7 +836,10 @@ glade_project_remove_pixmap		(GladeProject  *project,
 {
   GList *element;
 
-  g_return_if_fail (filename != NULL);
+  if (!filename)
+    return;
+
+  g_print ("Removing pixmap: %s\n", filename);
 
   element = project->pixmap_filenames;
   while (element)
@@ -1201,281 +1236,460 @@ glade_project_set_translatable_strings_file (GladeProject  *project,
 }
 
 
+/*
+ * Loading Project Options.
+ */
+typedef enum {
+    PARSER_OPTION,
+    PARSER_UNKNOWN
+} OptionsParserState;
+
+typedef struct _GladeOptionsParseState GladeOptionsParseState;
+struct _GladeOptionsParseState {
+  OptionsParserState state;
+
+  GladeProject *project;
+
+  gchar *base_directory;
+
+  GString *option_name;
+  GString *option_value;
+};
+
+
+/* This sets one project option, as it is loaded by the SAX parser. */
 void
-glade_project_load_options (GladeProject *project,
-			    GbWidgetSetArgData * data)
+glade_project_load_option (GladeOptionsParseState *state)
 {
-  gchar *element, *cdata;
-  gint element_index, cdata_index;
-  gchar *base;
+  GladeProject *project;
+  gchar *base, *option_name, *option_value;
 
-  base = g_dirname (project->xml_filename);
+  project = state->project;
+  base = state->base_directory;
+  option_name = state->option_name->str;
+  option_value = state->option_value->str;
 
-  /* Set some default options, which are different from those set in new(). */
-  project->gnome_support = TRUE;
-  project->gettext_support = TRUE;
+#if 0
+  g_print ("Setting Option:'%s' to '%s'\n", option_name, option_value);
+#endif
 
-  for (;;)
+  if (!strcmp (option_name, "name"))
     {
-      load_element (data, &element_index, &cdata_index);
-      element = data->buffer.ptr + element_index;
-      cdata = data->buffer.ptr + cdata_index;
-
-      if (data->status != GLADE_STATUS_OK)
-	{
-	  if (data->status == GLADE_STATUS_START_TAG_EXPECTED
-	      && data->token_type == GB_TOKEN_END_TAG
-	      && (!strcmp (data->buffer.ptr + data->token, "project")))
-	    {
-	      data->status = GLADE_STATUS_OK;
-	      break;
-	    }
-	  MSG ("Load error");
-	  break;
-	}
-
-      if (!strcmp (element, "name"))
-	{
-	  g_free (project->name);
-	  project->name = glade_util_copy_string (cdata);
-	}
-      else if (!strcmp (element, "program_name"))
-	{
-	  g_free (project->program_name);
-	  project->program_name = glade_util_copy_string (cdata);
-	}
-      else if ((!strcmp (element, "directory"))
-	       || (!strcmp (element, "project_dir"))) /* Old name. */
-	{
- 	  g_free (project->directory);
- 	  project->directory = glade_util_make_absolute_path (base, cdata);
-	}
-      else if ((!strcmp (element, "source_directory"))
-	       || (!strcmp (element, "srcdir"))) /* Old name. */
-	{
- 	  g_free (project->source_directory);
- 	  project->source_directory = glade_util_make_absolute_path (base,
-								     cdata);
-	}
-      else if (!strcmp (element, "pixmaps_directory"))
-	{
- 	  g_free (project->pixmaps_directory);
- 	  project->pixmaps_directory = glade_util_make_absolute_path (base,
-								      cdata);
-	}
-      else if (!strcmp (element, "language"))
-	{
-	  if (!glade_project_set_language_name (project, cdata))
-	    g_warning ("Invalid source language");
-	}
-      else if (!strcmp (element, "gnome_support"))
-	{
-	  gboolean gnome_support = load_parse_bool (data, cdata);
+      g_free (project->name);
+      project->name = glade_util_copy_string (option_value);
+    }
+  else if (!strcmp (option_name, "program_name"))
+    {
+      g_free (project->program_name);
+      project->program_name = glade_util_copy_string (option_value);
+    }
+  else if (!strcmp (option_name, "directory") && base)
+    {
+      g_free (project->directory);
+      project->directory = glade_util_make_absolute_path (base, option_value);
+    }
+  else if (!strcmp (option_name, "source_directory") && base)
+    {
+      g_free (project->source_directory);
+      project->source_directory = glade_util_make_absolute_path (base,
+								 option_value);
+    }
+  else if (!strcmp (option_name, "pixmaps_directory") && base)
+    {
+      g_free (project->pixmaps_directory);
+      project->pixmaps_directory = glade_util_make_absolute_path (base,
+								  option_value);
+    }
+  else if (!strcmp (option_name, "language"))
+    {
+      if (!glade_project_set_language_name (project, option_value))
+	g_warning ("Invalid source language");
+    }
+  else if (!strcmp (option_name, "gnome_support"))
+    {
+      gboolean gnome_support = load_parse_bool (NULL, option_value);
 #ifndef	USE_GNOME
-	  if (gnome_support == TRUE)
-	    {
-	      g_warning ("Glade has been compiled without support for Gnome.");
-	      data->status = GLADE_STATUS_ERROR;
-	    }
-#endif
-	  project->gnome_support = gnome_support;
-	}
-      else if (!strcmp (element, "gnome_db_support"))
+      if (gnome_support == TRUE)
 	{
-	  gboolean gnome_db_support = load_parse_bool (data, cdata);
+	  g_warning ("Glade has been compiled without support for Gnome.");
+	}
+#endif
+      project->gnome_support = gnome_support;
+    }
+  else if (!strcmp (option_name, "gnome_db_support"))
+    {
+      gboolean gnome_db_support = load_parse_bool (NULL, option_value);
 #ifndef	USE_GNOME_DB
-	  if (gnome_db_support == TRUE)
-	    {
-	      g_warning ("Glade has been compiled without support for Gnome DB.");
-	      data->status = GLADE_STATUS_ERROR;
-	    }
+      if (gnome_db_support == TRUE)
+	{
+	  g_warning ("Glade has been compiled without support for Gnome DB.");
+	}
 #endif
-	  project->gnome_db_support = gnome_db_support;
-	}
-      else if ((!strcmp (element, "gettext_support"))
-	       || (!strcmp (element, "i18n_strings"))) /* Old name. */
-	{
-	  project->gettext_support = load_parse_bool (data, cdata);
-	}
-      else if (!strcmp (element, "use_widget_names"))
-	{
-	  project->use_widget_names = load_parse_bool (data, cdata);
-	}
-      else if (!strcmp (element, "output_main_file"))
-	{
-	  project->output_main_file = load_parse_bool (data, cdata);
-	}
-      else if (!strcmp (element, "output_support_files"))
-	{
-	  project->output_support_files = load_parse_bool (data, cdata);
-	}
-      else if (!strcmp (element, "output_build_files"))
-	{
-	  project->output_build_files = load_parse_bool (data, cdata);
-	}
-      else if (!strcmp (element, "backup_source_files"))
-	{
-	  project->backup_source_files = load_parse_bool (data, cdata);
-	}
-      else if (!strcmp (element, "gnome_help_support"))
-	{
-	  project->gnome_help_support = load_parse_bool (data, cdata);
-	}
-      else if (!strcmp (element, "main_source_file"))
-	{
-	  g_free (project->main_source_file);
-	  project->main_source_file = glade_util_copy_string (cdata);
-	}
-      else if (!strcmp (element, "main_header_file"))
-	{
-	  g_free (project->main_header_file);
-	  project->main_header_file = glade_util_copy_string (cdata);
-	}
-      else if (!strcmp (element, "handler_source_file"))
-	{
-	  g_free (project->handler_source_file);
-	  project->handler_source_file = glade_util_copy_string (cdata);
-	}
-      else if (!strcmp (element, "handler_header_file"))
-	{
-	  g_free (project->handler_header_file);
-	  project->handler_header_file = glade_util_copy_string (cdata);
-	}
-      else if (!strcmp (element, "support_source_file"))
-	{
-	  g_free (project->support_source_file);
-	  project->support_source_file = glade_util_copy_string (cdata);
-	}
-      else if (!strcmp (element, "support_header_file"))
-	{
-	  g_free (project->support_header_file);
-	  project->support_header_file = glade_util_copy_string (cdata);
-	}
-
-      else if (!strcmp (element, "output_translatable_strings"))
-	{
-	  project->output_translatable_strings = load_parse_bool (data, cdata);
-	}
-      else if (!strcmp (element, "translatable_strings_file"))
-	{
-	  g_free (project->translatable_strings_file);
- 	  project->translatable_strings_file = glade_util_make_absolute_path (base, cdata);
-	}
-
-      else
-	{
-	  g_warning ("Invalid element in project: %s\n", element);
-	}
-
-      if (data->status != GLADE_STATUS_OK)
-	break;
+      project->gnome_db_support = gnome_db_support;
+    }
+  else if (!strcmp (option_name, "gettext_support"))
+    {
+      project->gettext_support = load_parse_bool (NULL, option_value);
+    }
+  else if (!strcmp (option_name, "use_widget_names"))
+    {
+      project->use_widget_names = load_parse_bool (NULL, option_value);
+    }
+  else if (!strcmp (option_name, "output_main_file"))
+    {
+      project->output_main_file = load_parse_bool (NULL, option_value);
+    }
+  else if (!strcmp (option_name, "output_support_files"))
+    {
+      project->output_support_files = load_parse_bool (NULL, option_value);
+    }
+  else if (!strcmp (option_name, "output_build_files"))
+    {
+      project->output_build_files = load_parse_bool (NULL, option_value);
+    }
+  else if (!strcmp (option_name, "backup_source_files"))
+    {
+      project->backup_source_files = load_parse_bool (NULL, option_value);
+    }
+  else if (!strcmp (option_name, "gnome_help_support"))
+    {
+      project->gnome_help_support = load_parse_bool (NULL, option_value);
+    }
+  else if (!strcmp (option_name, "main_source_file"))
+    {
+      g_free (project->main_source_file);
+      project->main_source_file = glade_util_copy_string (option_value);
+    }
+  else if (!strcmp (option_name, "main_header_file"))
+    {
+      g_free (project->main_header_file);
+      project->main_header_file = glade_util_copy_string (option_value);
+    }
+  else if (!strcmp (option_name, "handler_source_file"))
+    {
+      g_free (project->handler_source_file);
+      project->handler_source_file = glade_util_copy_string (option_value);
+    }
+  else if (!strcmp (option_name, "handler_header_file"))
+    {
+      g_free (project->handler_header_file);
+      project->handler_header_file = glade_util_copy_string (option_value);
+    }
+  else if (!strcmp (option_name, "support_source_file"))
+    {
+      g_free (project->support_source_file);
+      project->support_source_file = glade_util_copy_string (option_value);
+    }
+  else if (!strcmp (option_name, "support_header_file"))
+    {
+      g_free (project->support_header_file);
+      project->support_header_file = glade_util_copy_string (option_value);
+    }
+  
+  else if (!strcmp (option_name, "output_translatable_strings"))
+    {
+      project->output_translatable_strings = load_parse_bool (NULL, option_value);
+    }
+  else if (!strcmp (option_name, "translatable_strings_file") && base)
+    {
+      g_free (project->translatable_strings_file);
+      project->translatable_strings_file = glade_util_make_absolute_path (base, option_value);
     }
 
-  /* Check that some basic options are set. */
-  if (project->directory == NULL)
-    project->directory = g_strdup (base);
-  if (project->source_directory == NULL)
-    project->source_directory = glade_util_make_absolute_path (base, "src");
-  if (project->pixmaps_directory == NULL)
-    project->pixmaps_directory = glade_util_make_absolute_path (base,
-								"pixmaps");
+  else
+    {
+      g_warning ("Unknown project option: %s\n", option_name);
+    }
+}
 
+
+static xmlEntityPtr
+glade_options_parser_get_entity(GladeOptionsParseState *state, const xmlChar *name)
+{
+    return xmlGetPredefinedEntity(name);
+}
+
+static void
+glade_options_parser_warning(GladeOptionsParseState *state, const char *msg, ...)
+{
+    va_list args;
+
+    va_start(args, msg);
+    g_logv("XML", G_LOG_LEVEL_WARNING, msg, args);
+    va_end(args);
+}
+
+static void
+glade_options_parser_error(GladeOptionsParseState *state, const char *msg, ...)
+{
+    va_list args;
+
+    va_start(args, msg);
+    g_logv("XML", G_LOG_LEVEL_CRITICAL, msg, args);
+    va_end(args);
+}
+
+static void
+glade_options_parser_fatal_error(GladeOptionsParseState *state, const char *msg, ...)
+{
+    va_list args;
+
+    va_start(args, msg);
+    g_logv("XML", G_LOG_LEVEL_ERROR, msg, args);
+    va_end(args);
+}
+
+static void
+glade_options_parser_start_document(GladeOptionsParseState *state)
+{
+  state->state = PARSER_UNKNOWN;
+
+  state->option_name = g_string_sized_new (128);
+  state->option_value = g_string_sized_new (1024);
+}
+
+static void
+glade_options_parser_end_document(GladeOptionsParseState *state)
+{
+  g_string_free (state->option_name, TRUE);
+  g_string_free (state->option_value, TRUE);
+}
+
+static void
+glade_options_parser_start_element(GladeOptionsParseState *state,
+			   const xmlChar *name, const xmlChar **attrs)
+{
+  g_string_assign (state->option_name, name);
+  g_string_truncate (state->option_value, 0);
+
+  state->state = PARSER_OPTION;
+}
+
+static void
+glade_options_parser_end_element(GladeOptionsParseState *state, const xmlChar *name)
+{
+  if (state->state != PARSER_OPTION)
+    return;
+
+  glade_project_load_option (state);
+
+  state->state = PARSER_UNKNOWN;
+}
+
+static void
+glade_options_parser_characters(GladeOptionsParseState *state, const xmlChar *chars, int len)
+{
+    switch (state->state) {
+    case PARSER_OPTION:
+	g_string_append_len (state->option_value, chars, len);
+	break;
+    default:
+	/* don't care about content in any other states */
+	break;
+    }
+}
+
+static xmlSAXHandler glade_options_parser = {
+    0, /* internalSubset */
+    0, /* isStandalone */
+    0, /* hasInternalSubset */
+    0, /* hasExternalSubset */
+    0, /* resolveEntity */
+    (getEntitySAXFunc)glade_options_parser_get_entity, /* getEntity */
+    0, /* entityDecl */
+    0, /* notationDecl */
+    0, /* attributeDecl */
+    0, /* elementDecl */
+    0, /* unparsedEntityDecl */
+    0, /* setDocumentLocator */
+    (startDocumentSAXFunc)glade_options_parser_start_document, /* startDocument */
+    (endDocumentSAXFunc)glade_options_parser_end_document, /* endDocument */
+    (startElementSAXFunc)glade_options_parser_start_element, /* startElement */
+    (endElementSAXFunc)glade_options_parser_end_element, /* endElement */
+    0, /* reference */
+    (charactersSAXFunc)glade_options_parser_characters, /* characters */
+    0, /* ignorableWhitespace */
+    0, /* processingInstruction */
+    (commentSAXFunc)0, /* comment */
+    (warningSAXFunc)glade_options_parser_warning, /* warning */
+    (errorSAXFunc)glade_options_parser_error, /* error */
+    (fatalErrorSAXFunc)glade_options_parser_fatal_error, /* fatalError */
+};
+
+
+
+
+void
+glade_project_load_options (GladeProject *project)
+{
+  gchar *filename, *base;
+
+  /* Check if the options file exists. If it doesn't we use defaults for
+     everything. */
+  filename = g_strdup_printf ("%sp", GladeSessionFile ? GladeSessionFile : project->xml_filename);
+  base = project->xml_filename ? g_dirname (project->xml_filename) : NULL;
+
+  if (glade_util_file_exists (filename))
+    {
+      GladeOptionsParseState state = { 0 };
+
+      state.project = project;
+      state.base_directory = base;
+
+      if (xmlSAXUserParseFile (&glade_options_parser, &state, filename) < 0)
+	{
+	  g_warning("document not well formed!");
+	}
+    }
+
+  /* Check that the directory options are set to defaults, but only if we
+     have a project filename set. We may not have one when loading a session
+     file. */
+  if (base)
+    {
+      if (project->directory == NULL)
+	project->directory = g_strdup (base);
+      if (project->source_directory == NULL)
+	project->source_directory = glade_util_make_absolute_path (base, "src");
+      if (project->pixmaps_directory == NULL)
+	project->pixmaps_directory = glade_util_make_absolute_path (base,
+								    "pixmaps");
+    }
+
+  g_free (filename);
   g_free (base);
 }
 
 
-void
-glade_project_save_options (GladeProject *project,
-			    GbWidgetGetArgData * data)
+/*
+ * Saving Project Options.
+ */
+static void
+save_option (GString *buffer, gint indent, gchar *tag_name, gchar *tag_value)
 {
+  gint i;
+
+  for (i = 0; i < indent; i++)
+    g_string_append (buffer, "  ");
+
+  g_string_append_printf (buffer, "<%s>%s</%s>\n",
+			  tag_name, tag_value ? tag_value : "", tag_name);
+}
+
+
+static void
+save_bool_option (GString *buffer, gint indent, gchar *tag_name,
+		  gboolean tag_value)
+{
+  save_option (buffer, indent, tag_name, tag_value ? "TRUE" : "FALSE");
+}
+
+
+GladeError*
+glade_project_save_options (GladeProject *project,
+			    FILE *fp)
+{
+  GladeError *error = NULL;
+  GString *buffer;
   gchar *base_dir, *dir, *file;
+  gint indent = 1, bytes_written;
 
-  save_start_tag (data, "project");
+  buffer = g_string_sized_new (1024);
 
-  save_string (data, "name", project->name);
-  save_string (data, "program_name", project->program_name);
+  g_string_append (buffer, "<glade-project>\n");
+
+  save_option (buffer, indent, "name", project->name);
+  save_option (buffer, indent, "program_name", project->program_name);
 
   /* All directories are saved relative to the xml file's directory. */
   base_dir = glade_util_dirname (project->xml_filename);
 
-  /* For now we will only use default values for C options which will not
-     affect Glade versions 0.5.0-0.5.5. */
+  /* We use defaults for most properties so only a few properties need to be
+     saved. */
 
   dir = glade_util_make_relative_path (base_dir, project->directory);
-  /*if (strcmp (dir, ""))*/
-    save_string (data, "directory", dir);
+  if (strcmp (dir, ""))
+    save_option (buffer, indent, "directory", dir);
   g_free (dir);
 
   dir = glade_util_make_relative_path (base_dir, project->source_directory);
-  /*if (strcmp (dir, "src"))*/
-    save_string (data, "source_directory", dir);
+  if (strcmp (dir, "src"))
+    save_option (buffer, indent, "source_directory", dir);
   g_free (dir);
 
   dir = glade_util_make_relative_path (base_dir, project->pixmaps_directory);
-  /*if (strcmp (dir, "pixmaps"))*/
-    save_string (data, "pixmaps_directory", dir);
+  if (strcmp (dir, "pixmaps"))
+    save_option (buffer, indent, "pixmaps_directory", dir);
   g_free (dir);
 
-  /*if (project->language != GLADE_LANGUAGE_C)*/
-    save_string (data, "language", GladeLanguages[project->language]);
-  /*if (!project->gnome_support)*/
-    save_bool   (data, "gnome_support", project->gnome_support);
+  if (project->language != GLADE_LANGUAGE_C)
+    save_option (buffer, indent, "language", GladeLanguages[project->language]);
+  if (!project->gnome_support)
+    save_bool_option (buffer, indent, "gnome_support", project->gnome_support);
 
   if (project->gnome_db_support)
-    save_bool   (data, "gnome_db_support", project->gnome_db_support);
+    save_bool_option (buffer, indent, "gnome_db_support", project->gnome_db_support);
 
 
   /*
    * C Options.
    */
-  /*if (!project->gettext_support)*/
-    save_bool   (data, "gettext_support", project->gettext_support);
+  if (!project->gettext_support)
+    save_bool_option (buffer, indent, "gettext_support", project->gettext_support);
   if (project->use_widget_names)
-    save_bool   (data, "use_widget_names", project->use_widget_names);
+    save_bool_option (buffer, indent, "use_widget_names", project->use_widget_names);
   if (!project->output_main_file)
-    save_bool   (data, "output_main_file", project->output_main_file);
+    save_bool_option (buffer, indent, "output_main_file", project->output_main_file);
   if (!project->output_support_files)
-    save_bool   (data, "output_support_files", project->output_support_files); 
+    save_bool_option (buffer, indent, "output_support_files", project->output_support_files); 
   if (!project->output_build_files)
-    save_bool   (data, "output_build_files", project->output_build_files);
+    save_bool_option (buffer, indent, "output_build_files", project->output_build_files);
   if (!project->backup_source_files)
-    save_bool   (data, "backup_source_files", project->backup_source_files);
+    save_bool_option (buffer, indent, "backup_source_files", project->backup_source_files);
   if (project->gnome_help_support)
-    save_bool   (data, "gnome_help_support", project->gnome_help_support);
+    save_bool_option (buffer, indent, "gnome_help_support", project->gnome_help_support);
 
   if (!project->main_source_file
       || strcmp (project->main_source_file, "interface.c"))
-    save_string (data, "main_source_file", project->main_source_file);
+    save_option (buffer, indent, "main_source_file", project->main_source_file);
   if (!project->main_header_file
       || strcmp (project->main_header_file, "interface.h"))
-    save_string (data, "main_header_file", project->main_header_file);
+    save_option (buffer, indent, "main_header_file", project->main_header_file);
   if (!project->handler_source_file
       || strcmp (project->handler_source_file, "callbacks.c"))
-    save_string (data, "handler_source_file", project->handler_source_file);
+    save_option (buffer, indent, "handler_source_file", project->handler_source_file);
   if (!project->handler_header_file
       || strcmp (project->handler_header_file, "callbacks.h"))
-    save_string (data, "handler_header_file", project->handler_header_file);
+    save_option (buffer, indent, "handler_header_file", project->handler_header_file);
   if (!project->support_source_file
       || strcmp (project->support_source_file, "support.c"))
-    save_string (data, "support_source_file", project->support_source_file);
+    save_option (buffer, indent, "support_source_file", project->support_source_file);
   if (!project->support_header_file
       || strcmp (project->support_header_file, "support.h"))
-    save_string (data, "support_header_file", project->support_header_file);
+    save_option (buffer, indent, "support_header_file", project->support_header_file);
 
   if (project->output_translatable_strings)
-    save_bool (data, "output_translatable_strings", TRUE);
+    save_bool_option (buffer, indent, "output_translatable_strings", TRUE);
   if (project->translatable_strings_file
       && project->translatable_strings_file[0])
     {
       file = glade_util_make_relative_path (base_dir, project->translatable_strings_file);
       if (strcmp (file, ""))
-	save_string (data, "translatable_strings_file", file);
+	save_option (buffer, indent, "translatable_strings_file", file);
       g_free (file);
     }
 
   g_free (base_dir);
 
-  save_end_tag (data, "project");
+  g_string_append (buffer, "</glade-project>\n");
+
+  bytes_written = fwrite (buffer->str, sizeof (gchar), buffer->len, fp);
+  if (bytes_written != buffer->len)
+    error = glade_error_new_system (_("Error writing project XML file\n"));
+
+  g_string_free (buffer, TRUE);
+
+  return error;
 }
 
 
@@ -1492,7 +1706,7 @@ gchar*
 glade_project_new_widget_name (GladeProject *project,
 			       const gchar  *base_name)
 {
-  char new_widget_name[128];
+  char new_widget_name[128], *id_start;
   gint widget_id, i, new_widget_name_len;
 
   /* Check we won't overflow the buffer. */
@@ -1505,6 +1719,12 @@ glade_project_new_widget_name (GladeProject *project,
     base_name += 5;
 
   strcpy (new_widget_name, base_name);
+
+  /* Remove any id number at the end of the name. */
+  id_start = glade_project_find_id (project, new_widget_name);
+  if (id_start)
+    *id_start = '\0';
+
   /* convert name to lower case (only normal ASCII chars) */
   new_widget_name_len = strlen (new_widget_name);
   for (i = 0; i < new_widget_name_len; i++)
@@ -1635,7 +1855,7 @@ glade_project_reserve_name (GladeProject *project,
 
 /* This returns the start of the ID number at the end of the widget name,
    or NULL if there is no number. */
-static const gchar*
+static gchar*
 glade_project_find_id (GladeProject *project,
 		       const gchar *name)
 {
@@ -1657,7 +1877,7 @@ glade_project_find_id (GladeProject *project,
   if (name[pos] == '\0')
     return NULL;
 
-  return &name[pos];
+  return (gchar*) &name[pos];
 }
 
 
