@@ -18,7 +18,8 @@
 
 #include <config.h>
 
-#include <gnome-db.h>
+#include <gtk/gtkmain.h>
+#include <libgnomedb/gnome-db-login-dialog.h>
 #include "../gb.h"
 
 /* Include the 21x21 icon pixmap for this widget, to be used in the palette */
@@ -32,7 +33,7 @@ static GbWidget gbwidget;
 
 static gchar *Title = "GnomeDbLoginDlg|GtkWindow::title";
 static gchar *Type = "GnomeDbLoginDlg|GtkWindow::type";
-static gchar *Position = "GnomeDbLoginDlg|GtkWindow::position";
+static gchar *Position = "GnomeDbLoginDlg|GtkWindow::window_position";
 static gchar *Modal = "GnomeDbLoginDlg|GtkWindow::modal";
 static gchar *DefaultWidth = "GnomeDbLoginDlg|GtkWindow::default_width";
 static gchar *DefaultHeight = "GnomeDbLoginDlg|GtkWindow::default_height";
@@ -42,6 +43,9 @@ static gchar *AutoShrink = "GnomeDbLoginDlg|GtkWindow::auto_shrink";
 static gchar *WMName = "GnomeDbLoginDlg|GtkWindow::wmclass_name";
 static gchar *WMClass = "GnomeDbLoginDlg|GtkWindow::wmclass_class";
 
+static gchar *Resizable = "GnomeDbLoginDlg|GtkWindow::resizable";
+static gchar *DestroyWithParent = "GnomeDbLoginDlg|GtkWindow::destroy_with_parent";
+static gchar *Icon = "GnomeDbLoginDlg|GtkWindow::icon";
 
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
@@ -60,13 +64,9 @@ gb_gnome_db_logindlg_new (GbWidgetNewData *data)
   GtkWidget *widget, *login;
   GList *elem;
 
-  login = gnome_db_login_new (NULL, NULL, NULL);
+  login = gnome_db_login_new (NULL);
 
-  widget = gnome_db_logindlg_new (GNOME_DB_LOGIN (login), _("Login"));
-
-  /* Stop the normal OK operation since it can cause a crash. */
-  elem = g_list_nth (GNOME_DIALOG (widget)->buttons, 0);
-  gtk_signal_disconnect_by_data (GTK_OBJECT (elem->data), widget);
+  widget = gnome_db_login_dialog_new (_("Login"));
 
   /* We connect a close signal handler which always returns TRUE so that
      the built-in close functionality is skipped. */
@@ -92,7 +92,8 @@ gb_gnome_db_logindlg_create_properties (GtkWidget * widget, GbWidgetCreateArgDat
 					Title, Type, Position, Modal,
 					DefaultWidth, DefaultHeight,
 					Shrink, Grow, AutoShrink,
-					WMName, WMClass);
+					WMName, WMClass,
+					Resizable, DestroyWithParent, Icon);
 }
 
 
@@ -108,7 +109,8 @@ gb_gnome_db_logindlg_get_properties (GtkWidget *widget, GbWidgetGetArgData * dat
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass);
+				     WMName, WMClass,
+				     Resizable, DestroyWithParent, Icon);
 }
 
 
@@ -124,7 +126,8 @@ gb_gnome_db_logindlg_set_properties (GtkWidget * widget, GbWidgetSetArgData * da
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass);
+				     WMName, WMClass,
+				     Resizable, DestroyWithParent, Icon);
 }
 
 
@@ -134,14 +137,11 @@ gb_gnome_db_logindlg_set_properties (GtkWidget * widget, GbWidgetSetArgData * da
  * Add commands to aid in editing a GnomeDbLoginDlg, with signals pointing to
  * other functions in this file.
  */
-/*
 static void
 gb_gnome_db_logindlg_create_popup_menu (GtkWidget * widget, GbWidgetCreateMenuData * data)
 {
 
 }
-*/
-
 
 
 /*
@@ -157,7 +157,7 @@ gb_gnome_db_logindlg_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
       gchar *title;
 
       title = GTK_WINDOW (widget)->title;
-      source_add (data, "  %s = gnome_db_logindlg_new (GNOME_DB_LOGIN (gnome_db_login_new (NULL, NULL, NULL)), %s);\n",
+      source_add (data, "  %s = gnome_db_logindlg_new (%s);\n",
 		  data->wname,
 		  title ? source_make_string (title, data->use_gettext)
 		        : "NULL");
@@ -170,7 +170,8 @@ gb_gnome_db_logindlg_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
 				   NULL, Type, Position, Modal,
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, AutoShrink,
-				   WMName, WMClass);
+				   WMName, WMClass,
+				   Resizable, DestroyWithParent, Icon);
 }
 
 
@@ -184,7 +185,8 @@ GbWidget*
 gb_gnome_db_logindlg_init ()
 {
   /* Initialise the GTK type */
-  gnome_db_logindlg_get_type();
+  volatile GtkType type;
+  type = gnome_db_login_dialog_get_type();
 
   /* Initialize the GbWidget structure */
   gb_widget_init_struct(&gbwidget);
@@ -199,9 +201,7 @@ gb_gnome_db_logindlg_init ()
   gbwidget.gb_widget_get_properties	= gb_gnome_db_logindlg_get_properties;
   gbwidget.gb_widget_set_properties	= gb_gnome_db_logindlg_set_properties;
   gbwidget.gb_widget_write_source	= gb_gnome_db_logindlg_write_source;
-/*
   gbwidget.gb_widget_create_popup_menu	= gb_gnome_db_logindlg_create_popup_menu;
-*/
 
   return &gbwidget;
 }
